/* Copyright © 2020 tyrolyean
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "avr.h"
#include "16550.h"
#include "structures.h"
#include "game.h"

#include <stdio.h>
static FILE stdout_16550 = FDEV_SETUP_STREAM(putchar_16550, NULL, 
	_FDEV_SETUP_WRITE);


void write_to_uart(uint8_t addr, uint8_t data){

	set_addr(addr);
	DATA_DDR_REG = 0xFF;
	DATA_REG = data;
	CTRL_REG &= ~(1<<WR_SHIFT) & ~(1<<CS_UART_SHIFT);

	_delay_us(BUS_HOLD_US);	/*Wait for the data and signal lanes to become stable*/

	CTRL_REG |= 1<<CS_UART_SHIFT | 1<<WR_SHIFT;
	return;
}

uint8_t read_from_uart(uint8_t addr){

	uint8_t data = 0x00;

	set_addr(addr);
	DATA_DDR_REG = 0x00;
	/* Sometimes the avr "forgets" to alter the port register for one cycle
	 * which usually isn't a problem but has lead to instability. Worthy 
	 * one clock cycle */
	DATA_REG = 0x00;
	CTRL_REG &= ~(1<<RD_SHIFT) & ~(1<<CS_UART_SHIFT);

	_delay_us(BUS_HOLD_US);	/* Wait for the data and signal lanes to become
	                         * stable*/
	data = PINF;
	CTRL_REG |= 1<<CS_UART_SHIFT | 1<<RD_SHIFT;
	return data;
}

void init_uart(){

	write_to_uart(UART_REG_LCR,0x83);
	write_to_uart(UART_REG_DLLS,0x03);
	write_to_uart(UART_REG_DLMS,0x00);
	write_to_uart(UART_REG_LCR,0x03);
	
	stdout = &stdout_16550;
	printf("INIT\r\n");	
	return;
}

int println(const char* output, ...){

        va_list arg;
        va_start (arg, output);
	vprintf(output,arg);
	printf("\n");
        return 0;

}


/* Writes a character to the 16550 uart for transmission with cr lf magic */
int putchar_16550(char var, FILE *stream __attribute__((unused))){
	static uint8_t column_cnt = 0;
	column_cnt++;
	if(var == ' ' && column_cnt > (DISPLAY_COLUMNS - 10)){
		putchar_16550('\n',NULL);
		return 0;
	}
	if(var == '\n'){
		column_cnt = 0x00;
		writechar_16550('\r');
	}
	
	writechar_16550(var);
	return 0;
}
/* Writes a character to the 16550 uart for transmission without cr lf magic */
void writechar_16550(char var){
	
	for(uint32_t i = 0; i < 1000000; i++ ){
		uint8_t received = read_from_uart(UART_REG_LSR);
		if((received & (1<<5))){
			break;
		}
	}
	
	write_to_uart(UART_REG_TXRX,var);
	return;
}

void routine_uart(){
	
	uint8_t received = read_from_uart(UART_REG_LSR);
	if(received & 0x01){
		received = read_from_uart(UART_REG_TXRX);
		ingest_user_char(received);
		if(received == '\r'){
			writechar_16550('\n');
		}
		writechar_16550(received); /* Echo back */
	}

	return;
}

